/*----------------------------------------------------------------------------
 *  Copyright (C) 20078 Maxim Integrated Products Incorporated, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Maxim Integrated Products
 *  shall not be used except as stated in the Maxim Integrated Products
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 */
    
/** 
 * \file LCD_8113_Funct.c
 * \brief Provides routines to drive the LCD on the DS8113 EV Kit board
 *
 * This file provides the routines for driving the LCD module contained on
 * the DS8113 Evaluation Kit board. This is a two-line by 20 character
 * display (LCM-S02002DSF).
 *
 */

#include "maxq2000.h"
#include "LCD_main.h"
#include "LCD_8113_Funct.h"

/* Define macros to set,clear and read maxq2000 GPIO pins used to interface with */
/* with the LCD module.                                                          */
#define set_bit(out_reg, bit_mask)      (out_reg|=bit_mask)
#define clear_bit(out_reg,bit_mask)     (out_reg&=(~bit_mask))
#define read_bit(in_reg,bit_mask)       (((in_reg&bit_mask)==bit_mask)?1:0)
#define configure_in(dir_reg,bit_mask)  (dir_reg &= (~bit_mask))
#define configure_out(dir_reg,bit_mask) (dir_reg|=bit_mask)

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* Initialize the LCD module                                                     */
/*                                                                               */

void LCD_Init(void)
{
    LCD_CntDir=0xE0;              // Establish LCD control pins as outputs
    LCD_DatDir=0x00;              // Establish LCD Data pins as inputs
    
    LCD_PUDelay();                // Delay to ensure LCD_Power-Up complete

    LCD_WRCmd(FnctSet);           // Write initialization command sequence to LCD
    LCD_WRCmd(DispCnt);
    LCD_WRCmd(DispClear);
    LCD_WRCmd(EntryMode);

}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* Write a string to the LCD module                                              */
/*                                                                               */

void LCD_WRStr(uint8_t LCD_Str[])
{
    int i=0;

    while((LCD_Str[i]!='\0')&&(i<20)){
      LCD_WRChr(LCD_Str[i]);      // Write first 20 characters
      i++;
    }

    if(LCD_Str[i]!='\0')
       LCD_WRCmd(Line2Ad);        // Set CGRAM address to the first character of the
                                  // second line of display
    while((LCD_Str[i]!='\0')&&(i<40)){
      LCD_WRChr(LCD_Str[i]);
      i++;
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* Write a single character to the LCD module                                    */
/*                                                                               */

void LCD_WRChr(char LCD_Chr)
{
    LCD_Busy();                   // Wait for not busy
    set_bit(LCD_Cnt,LCD_RS);      // Set RS high for character write
    LCD_DatDir=Outpin;            // Make LCD data pins all outputs
    LCD_DatOut=LCD_Chr;           // Output character to LCD
    set_bit(LCD_Cnt,LCD_EN);      // Strobe in data with enable signal
    LCD_ENDelay();                // ******* Delay ******
    clear_bit(LCD_Cnt,LCD_EN);
    LCD_ENDelay();                // ******* Delay ******
    clear_bit(LCD_Cnt,LCD_RS);    // Return RS signal to 0
    LCD_DatDir=Inpin;             // Return LCD data pins to all inputs
    LCD_DatOut = 0xFF;            // Re-enable weak pull-ups (output register bits high)

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* Write a command to the LCD module.                                            */
/*                                                                               */

void LCD_WRCmd(uint8_t LCD_Cmd)
{
    clear_bit(LCD_Cnt,LCD_EN);    // Make sure the LCD Enalble signal is low
    clear_bit(LCD_Cnt,LCD_RS);    // Clear the LCD Register Select and R/W lines
    clear_bit(LCD_Cnt,LCD_RW);
    LCD_Busy();                   // Make sure the display is not busy

    LCD_DatDir=Outpin;            // Make LCD data pins all outputs
    LCD_DatOut = LCD_Cmd;         // Output the specific instruction
    set_bit(LCD_Cnt,LCD_EN);      // Strobe in command with the Enable
    LCD_ENDelay();                // ******* Delay ******
    clear_bit(LCD_Cnt,LCD_EN);
    LCD_ENDelay();                // ******* Delay ******
    LCD_DatDir=Inpin;             // Make LCD data pins all inputs
    LCD_DatOut = 0xFF;            // Re-enable weak pull-ups (output register bits high)
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* Set the LCD cursor position to a specific location on the display             */
/*                                                                               */

int LCD_Curpos(uint8_t VPos, uint8_t HPos)
{
    int rtn;
    uint8_t Addr,Cmd ;

    // Check input range 1..2 line, 1..20 characters per line
    if (((VPos==0)||(VPos>2))||((HPos==0)||(HPos>20)))
      rtn = -1;
    else {
      if(VPos==2) Addr=(0x40+(HPos-1));
      else Addr=HPos-1;
      rtn=0;
      Cmd=Addr|0x80;
      LCD_WRCmd(Cmd);
    }
    return(rtn);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* Test the LCD Module to see if it is busy, (loop until not busy)               */
/*                                                                               */

void LCD_Busy(void)
{
    uint8_t LCD_Stat = 0x80;      // Set initial status value as busy

    LCD_DatDir=Inpin ;            // Make sure P2 pins are all inputs
    while (LCD_Stat & 0x80) {
      set_bit(LCD_Cnt,LCD_RW);    // Set R/W line high for read
      set_bit(LCD_Cnt,LCD_EN);    // Strobe read with enable pin
      LCD_ENDelay();              // ******* Delay ******
      LCD_Stat = LCD_DatIn;       // Read status byte
      LCD_ENDelay();              // ******* Delay ******
      clear_bit(LCD_Cnt,LCD_EN);  // Return EN and R/W lines to 0
      LCD_ENDelay();              // ******* Delay ******
      clear_bit(LCD_Cnt,LCD_RW);
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* Time delay for 2x20 LCD module (approx 50mS) to power-up and stabilize        */
/*         (1/16MHz*8000)*100                                                    */

void LCD_PUDelay(void)
{
    int i,j;

    for (i=0;i<=99;i++)
    {
      for (j=0;j<=7999;j++);
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* Time delay for 2x20 LCD module to allow 625 ns min Enable signal. At the      */
/* time this function was written, the minimum period required for E was         */
/* was estimated as no specification was available.                              */
/*             (1/16MHz*10)=625 ns                                               */
/*                                                                               */

void LCD_ENDelay(void)
{
    int i;
    
    for (i=0;i<10;i++);
    
}

