/*----------------------------------------------------------------------------
 *  Copyright (C) 20078 Maxim Integrated Products Incorporated, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Maxim Integrated Products
 *  shall not be used except as stated in the Maxim Integrated Products
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 */
    
/** 
 * \file Transaction8113.c
 * \brief Provides main function for DS8113 Smart Card Transaction program
 *
 * This file provides the main function for a DS8113 Smart Cart Transaction
 * program that runs on the DS8113 Evaluation Kit board. The LCD on the board
 * and switch SW4 on the board provide the user interface. This program contains
 * functions required to initiate transaction operations on an ACOS3 Smart Card
 * using its operating system command APDUs. Functions for Debit, Credit, and Card
 * Initialization are implemented. The serial port is used to display some status
 * messages at 115200 baud over the RS-232 channel, but these status messages are
 * not required for the application.
 *
 */

#include <stdio.h>
#include <string.h>
#include <inmaxq.h>
#include <maxq2000.h>
#include "lib8113.h"
#include "LCD_8113_Funct.h"

#define set_bit(out_reg, bit_mask)      (out_reg|=bit_mask)
#define clear_bit(out_reg,bit_mask)     (out_reg&=(~bit_mask))
#define read_bit(in_reg,bit_mask)       (((in_reg&bit_mask)==bit_mask)?1:0)
#define configure_in(dir_reg,bit_mask)  (dir_reg &= (~bit_mask))
#define configure_out(dir_reg,bit_mask) (dir_reg|=bit_mask)

// Called-function prototypes
void initSerial(void);
void delay(int secs);
void doTrans(int slot);
void transError(void);
void dispTrans(void);
uint16_t transEnd(void);
uint16_t transATR(void);
uint8_t initCard(void);
uint8_t selDelay(int secs);
uint16_t selectFile(void);
uint16_t readRecords(void);
uint16_t transDebit(void);
uint16_t transCredit(void);
void setIV(void(*isr)(void));

uint8_t LCD_Str[42];

//
/* ************************************************************************** */
/* ************************************************************************** */
// Enable module 3 and globally enable interrupts
//
void EnableInterrupts(void)
{
    IMR=0x8;
    IC=1;
}

/* ************************************************************************** */
/* ************************************************************************** */
// Serial interrupt handling function
//
void Maxq20_interrupt(void) __interrupt
{
    if(IIR&0x8)
    {
        stopbit_interrupt();
    }
}

/* ************************************************************************** */
/* ************************************************************************** */
//
// Perform the ACOS3 transactions
//
void main(void)
{
    int slot = 0;
    int i, j ;
    uint8_t val, temp, *string;
    uint16_t val_16, temp16;

    setIV(Maxq20_interrupt);
    EnableInterrupts();
    __enable_interrupt();

    // Delay to let things stabilize
    for (i=0;i<100;i++){           
      for (j=0;j<1600;j++); }

    // Initialize the serial port, and output a startup message
    initSerial();
    puts("\n\nACOS3 Transaction Demo Example\n");
    string = dssc_version();
    printf("DS8113 Version: %s\n\n",string);

    // Initialize the DS8113 board
    temp16 = dssc_init();

    // Initialize LCD Module, and display 2-line message
    LCD_Init();
    strcpy(LCD_Str,"       DS8113");          // Create first line of LCD text
    temp = LCD_Curpos(1,1);                   // Position cursor @ 1 line 1
    if (temp == 0) LCD_WRStr(LCD_Str);        // Write string to LCD
    else printf("\n!! LCD cursor address out of range !!\n");
    strcpy(LCD_Str,"Smart Card Interface");   // Create 2nd line of LCD text
    temp = LCD_Curpos(2,1);                   // Position cursor @ 1 line 2
    if (temp == 0) LCD_WRStr(LCD_Str);        // Write string to LCD
    else printf("\n!! LCD cursor address out of range !!\n");
    temp = LCD_Curpos(2,20);                  // Return cussor @ 20 line 2

    delay(2);                                 // Delay for user to read message

    // Start main program loop
    while (1)
    {
      // Output initial instructions on LCD
      strcpy(LCD_Str, "Insert smart-card");
      LCD_WRCmd(DispClear);
      temp = LCD_Curpos (1,1);
      if (temp == 0) LCD_WRStr(LCD_Str);
      else printf("\n!! LCD cursor address out of range !!\n");
      strcpy(LCD_Str, "into socket below.");
      temp = LCD_Curpos (2,1);
      if (temp == 0) LCD_WRStr(LCD_Str);
      else printf("\n!! LCD cursor address out of range !!\n");

      printf("Waiting for smartcard presence\n");

      // Wait for card to be inserted
      while (1)
      {
        val_16 = dssc_checkpresence();
        if (val_16 == 1) {
          slot = 1;
          break ; } // end if val_16
      } // end while(1) card insert loop
                
      printf("Found smartcard.\n");

      // Start main transaction loop
      doTrans(slot);

      // Output ending message on LCD
      temp = transEnd(); 

      printf("Waiting for smartcard removal.\n");
  		
      // Wait for card to be removed
      do
        {val_16 = dssc_checkpresence(); }
      while( val_16 == 1 ) ;

      printf("Smartcard removed.\n");

    } // end while (1) main program loop

    // The following code should never be executed
    // It is just an emergency catch.
    printf("Exiting.\n");
    while (1);
}

/* ************************************************************************** */
/* ************************************************************************** */
//
// Perform the ACOS3 transactions
//
void doTrans(int slot)
{

  int i, tmp ;

  // Set up for transaction and get an Answer To Reset
  i = dssc_powerup(POWERUP_5V);
  if (i < 0) {
    printf("An error occurred at card power-up.\n");
    transError(); }
	
  // Get card ATR
  i = transATR() ;
  if (i < 0) {
    printf("An error occurred during the ATR.\n");
    transError(); }
  else printf("The card returned a good Answer To Reset.\n\n");

  // Display the current transaction Count and Balance
  i = selectFile();               // Select the Personalization file
  i = readRecords();              // Read the records containing Count and Balance
  dispTrans();                    // Display the results

  // Create main transaction processing loop
  while (dssc_checkpresence())    // Use presence detect as loop control
  {
    if (i >= 0)
    {
      // Cycle through tranction type options on LCD
      strcpy(LCD_Str, "For a Debit        ");
      LCD_WRCmd(DispClear);
      tmp = LCD_Curpos (1,1);
      if (tmp == 0) LCD_WRStr(LCD_Str);
      else printf("\n!! LCD cursor address out of range !!\n");
      strcpy(LCD_Str, "press SW4 now.     ");
      tmp = LCD_Curpos (2,1);
      if (tmp == 0) LCD_WRStr(LCD_Str);
      else printf("\n!! LCD cursor address out of range !!\n");
      tmp = selDelay(2);
      if (tmp == 1) {             // Perform a Debit transaction
        i = transDebit();
	printf("\nThe card was debited $10.00\n");
	break ; }

      if (!(dssc_checkpresence())) break; // Make sure the card is still inserted
      strcpy(LCD_Str, "For a Credit       ");
      tmp = LCD_Curpos (1,1);
      if (tmp == 0) LCD_WRStr(LCD_Str);
      else printf("\n!! LCD cursor address out of range !!\n");
      strcpy(LCD_Str, "press SW4 now.     ");
      tmp = LCD_Curpos (2,1);
      if (tmp == 0) LCD_WRStr(LCD_Str);
      else printf("\n!! LCD cursor address out of range !!\n");
      tmp = selDelay(2);
      if (tmp == 1) {             // Perform a Credit transaction
        i = transCredit();
	printf("\nThe card was credited $10.00\n");
	break ; }

      if (!(dssc_checkpresence())) break;   // Make sure the card is still inserted
      strcpy(LCD_Str, "For Initialization ");
      tmp = LCD_Curpos (1,1);
      if (tmp == 0) LCD_WRStr(LCD_Str);
      else printf("\n!! LCD cursor address out of range !!\n");
      strcpy(LCD_Str, "press SW4 now.     ");
      tmp = LCD_Curpos (2,1);
      if (tmp == 0) LCD_WRStr(LCD_Str);
      else printf("\n!! LCD cursor address out of range !!\n");
      tmp = selDelay(2);
      if (tmp == 1) {             // Initialize the card
        i = initCard();
        printf("\nThe card was initialized.\n");
	break ; }
      } else {
        transError();
        break ;
      } // end IF i > 1
  	
    } // end while transloop 

    printf("\nThe transaction ended.\n");
    dssc_powerdown();

    return;

} // end doTrans()
//

/* ************************************************************************** */
/* ************************************************************************** */
// Create a delay of "secs" seconds assuming a 13.5 MHz crystal
//
void delay(int secs)
{
  int i, j, k ;

  for (i=0;i<(secs*150);i++) {
    for (j=0;j<999;j++); {
      for (k=0;k<13499;k++);
    }
  }
}

/* ************************************************************************** */
/* ************************************************************************** */
// Create a delay of "secs" seconds assuming a 14.7 MHz crystal but testing
// for a press of SW4
//

uint8_t selDelay(int secs)
{

  int i, j, k, m ;

  m = secs * 150;
  i = 0;
  while ((i < m) && (read_bit(PI0, SW4_IN)))
  {
    for (j=0;j<999;j++); {
      for (k=0;k<13499;k++);
    }
  i++ ;
  }
  if (i >= m) return 0;
  else return 1;
}




