/*----------------------------------------------------------------------------
 *  Copyright (C) 2007 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 */

/** 
 * \file lib8113.c
 * \brief Interface library for DS8113 smartcard interface chips
 *
 * This library contains routines required to communicate with smartcards
 * via an 8113 while conforming to:
 * ISO 7816
 * EMV 4.1
 */

#include <maxq2000.h>
#include <inmaxq.h>
#include <string.h>  //included for memcpy function
#include "lib8113.h" //ds8113 emv library header file

//The following defines introduced for code readability
#define  COLD_RESET 1    //indicates cold reset
#define  WARM_RESET 2    //indicates warm reset

#define  SPECIFIC_MODE_USE_DEFAULT_F_D   1      /* Use default iso7816  F and D value */
#define  SPECIFIC_MODE_USE_TA1_F_D       2      /* Use Card ATR TA1 byte F and D value */
#define  NEGOTIABLE_MODE                 3      /* Card is in Negotiable mode */

#define T1_BLOCK_RETRY    3     /* Number of retries */

#define NONIBLOCK   0x80        /* Non I Block indicator */
#define SBLOCK      0x40        /* S Block indicator     */
#define MOREBIT     0x20        /* More bit indicator    */

#define SBLOCKFUNCMASK    0x3F    /* S-block function mask */
    
//Generates SBLOCK PCB BYTE
#define CODESBLOCK(x)        (unsigned char)((x)|NONIBLOCK|SBLOCK) 

//Generates RBLOCK PCB BYTE
#define CODERBLOCK(nr,rc)   (unsigned char)(NONIBLOCK|((nr) << 4)|(rc))

//Generates IBLOCK PCB BYTE
#define CODEIBLOCK(ns,mb)   (unsigned char)(((ns) << 6)|((mb) << 5))

//Returns TRUE if pcb belongs to I BLOCK
#define ISIBLOCK(pcb)    (!((pcb) & NONIBLOCK))            

//Returns TRUE if pcb belongs to R BLOCK
#define ISRBLOCK(pcb)    ((pcb & NONIBLOCK) && (!(pcb & SBLOCK))&&(!(pcb&MOREBIT)))    

//Returns TRUE if pcb belongs to S BLOCK
#define ISSBLOCK(pcb)    (((pcb) & NONIBLOCK) && ((pcb) & SBLOCK))    

#define NR(pcb)        (((pcb) & 0x10) >> 4)   /* Retrieves R_Block Sequence number from pcb*/
#define NS(pcb)        (((pcb) & 0x40) >> 6)   /* Retrieves I_Block Sequence number from pcb*/

#define RERR(pcb)     ((pcb) & RERR_MASK)       /* Retrieves Error field in R-blck pcb  */
#define MORE(pcb)     (((pcb) & MOREBIT) >> 5)  /* Retrieves More data bit info from pcb*/
#define SBLOCKFUNC(pcb)    ((pcb) & SBLOCKFUNCMASK)  /* Retrieves S-block function code */

#define SREQUEST_ABORT    0xC2 /* S-block Abort operation function code */
#define SREQUEST_IFS      0xC1 /* S-block IFS Request function code */
#define SREQUEST_WTX      0xC3 /* S-block WTX Request function code */

#define SRESPONSE_IFS     0xE1 /* S-block IFS Response function code */
#define SRESPONSE_WTX     0xE3 /* S-block WTX Response function code */
#define SRESPONSE_ABORT   0xE2 /* S-block Abort Response function code */

#define generateEDC(onebyte,value) (value^onebyte) //Computes one byte checksum 

//define macros to set,clear and read maxq2000 GPIO pin values
#define set_bit(out_reg, bit_mask)      (out_reg|=bit_mask)
#define clear_bit(out_reg,bit_mask)     (out_reg&=(~bit_mask))
#define read_bit(in_reg,bit_mask)       (((in_reg&bit_mask)==bit_mask)?1:0)
#define configure_in(dir_reg,bit_mask)  (dir_reg &= (~bit_mask))
#define configure_out(dir_reg,bit_mask) (dir_reg|=bit_mask)

uint8_t  card_workingBuffer[260];  // Card Working buffer Used for Storing ATR, T0 and 
uint8_t  ATRLength;                // T=1 transaction

#define DIRECT_CONVENTION  0
#define INVERSE_CONVENTION 1

#define MAX_BLOCK_LEN 260

uint8_t card_convention;

// Get Response APDU request header
uint8_t  GET_RESPONSE[5]={0,0xC0,0,0,0};

const __code uint8_t inverse_of[256]={

                          0xff, 0x7f, 0xbf, 0x3f, 0xdf, 0x5f, 0x9f, 0x1f, 0xef, 0x6f,\
                          0xaf, 0x2f, 0xcf, 0x4f, 0x8f, 0xf, 0xf7, 0x77, 0xb7, 0x37, \
                          0xd7, 0x57, 0x97, 0x17, 0xe7, 0x67, 0xa7, 0x27, 0xc7, 0x47,\
                          0x87, 0x7,  0xfb, 0x7b, 0xbb, 0x3b, 0xdb, 0x5b, 0x9b, 0x1b,\
                          0xeb, 0x6b, 0xab, 0x2b, 0xcb, 0x4b, 0x8b, 0xb, 0xf3, 0x73, \
                          0xb3, 0x33, 0xd3, 0x53, 0x93, 0x13, 0xe3, 0x63, 0xa3, 0x23,\
                          0xc3, 0x43, 0x83, 0x3,  0xfd, 0x7d, 0xbd, 0x3d, 0xdd, 0x5d,\
                          0x9d, 0x1d, 0xed, 0x6d, 0xad, 0x2d, 0xcd, 0x4d, 0x8d, 0xd, \
                          0xf5, 0x75, 0xb5, 0x35, 0xd5, 0x55, 0x95, 0x15, 0xe5, 0x65,\
                          0xa5, 0x25, 0xc5, 0x45, 0x85, 0x5, 0xf9, 0x79, 0xb9, 0x39, \
                          0xd9, 0x59, 0x99, 0x19, 0xe9, 0x69, 0xa9, 0x29, 0xc9, 0x49,\
                          0x89, 0x9, 0xf1,  0x71, 0xb1, 0x31, 0xd1, 0x51, 0x91, 0x11,\
                          0xe1, 0x61, 0xa1, 0x21, 0xc1, 0x41, 0x81, 0x1, 0xfe, 0x7e, \
                          0xbe, 0x3e, 0xde, 0x5e, 0x9e, 0x1e, 0xee, 0x6e, 0xae, 0x2e,\
                          0xce, 0x4e, 0x8e, 0xe,  0xf6, 0x76, 0xb6, 0x36, 0xd6, 0x56,\
                          0x96, 0x16, 0xe6, 0x66, 0xa6, 0x26, 0xc6, 0x46, 0x86, 0x6, \
                          0xfa, 0x7a, 0xba, 0x3a, 0xda, 0x5a, 0x9a, 0x1a, 0xea, 0x6a,\
                          0xaa, 0x2a, 0xca, 0x4a, 0x8a, 0xa, 0xf2, 0x72, 0xb2, 0x32, \
                          0xd2, 0x52, 0x92, 0x12, 0xe2, 0x62, 0xa2, 0x22, 0xc2, 0x42,\
                          0x82, 0x2,  0xfc, 0x7c, 0xbc, 0x3c, 0xdc, 0x5c, 0x9c, 0x1c,\
                          0xec, 0x6c, 0xac, 0x2c, 0xcc, 0x4c, 0x8c, 0xc, 0xf4, 0x74, \
                          0xb4, 0x34, 0xd4, 0x54, 0x94, 0x14, 0xe4, 0x64, 0xa4, 0x24,\
                          0xc4, 0x44, 0x84, 0x4, 0xf8, 0x78, 0xb8, 0x38, 0xd8, 0x58, \
                          0x98, 0x18, 0xe8, 0x68, 0xa8, 0x28, 0xc8, 0x48, 0x88, 0x8, \
                          0xf0, 0x70, 0xb0, 0x30, 0xd0, 0x50, 0x90, 0x10, 0xe0, 0x60,\
                          0xa0, 0x20, 0xc0, 0x40, 0x80, 0x0};

/*
uint8_t parity_of_inverse_of[256]={
                          1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0,\
                          1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,\
                          1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0,\
                          1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,\
                          1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1,\
                          0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1,\
                          1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1,\
                          0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,\
                          1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0,\
                          1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0,\
                          0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1,\
                          0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,\
                          1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1};
*/

const __code uint8_t parity_of_direct[256]={
                          0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1,\
                          0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,\
                          0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1,\
                          0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,\
                          0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0,\
                          1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0,\
                          0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0,\
                          1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,\
                          0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1,\
                          0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1,\
                          1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0,\
                          1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1,\
                          0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0};

uint8_t  TMode;   // T=0 or T=1
uint8_t  ATRmode; // specifies whether Card is in SPECIFIC_MODE or NEGOTIABLE_MODE
uint32_t WWT;     // Work wait time
uint32_t CWT;     // Character wait time
uint32_t BWT;     // Block wait time
uint8_t  EDCtype;
uint8_t  NAD;     // Node address byte to use when communicating in T=1
uint8_t  IFSC;    // Maximum segment length when sending to card in T=1
uint8_t  GTR;     // Guard Time value.
uint8_t  D=1;       // Bit rate adjustment factor

uint8_t  IFSD_Negotiated=0; //Flag indicates whether IFSD negotiation happened
uint8_t  IFD_Seq;           // Contains next Terminal I-block sequence number
uint8_t  ICC_Seq;           // Contains next Card I-block sequence number

uint8_t  wtx=0;                 // Maintains work time extension value
uint8_t  t1_parity_error=0;     // global status flag used to share parity error status
uint8_t  t1_sblock_sent=0;      // global status flag used to maintain t1 protocol state.

unsigned char serial1_tbe_flag,serial1_rbf_flag, timer_configuration;

uint16_t timer1_iteration,timer1_num_of_iteration, timer1_initial_load;
uint16_t timer2_iteration,timer2_num_of_iteration, timer2_initial_load;

// Write block function sends t1 block to card
void writeBlock(uint8_t NAD, uint8_t PCB, uint8_t length, uint8_t *buffer);

// internal functions declarations
int16_t  dssc_ATRsequence(uint8_t reset_type);

#define start_timer21() set_bit(T2CNA1, T2CNA1_TR2);
#define start_timer22() set_bit(T2CNA2, T2CNA2_TR2);

#define ENABLED  1
#define DISABLED 0

#define set_serial1_tbe_flag()   serial1_tbe_flag=ENABLED;
#define clear_serial1_tbe_flag() serial1_tbe_flag=DISABLED;

#define set_serial1_rbf_flag()   serial1_rbf_flag=ENABLED;
#define clear_serial1_rbf_flag() serial1_rbf_flag=DISABLED;

#define disable_stopbit_interrupt()    {SMD1&=0xFB;}
#define enable_stopbit_interrupt()     {SMD1|=0x04;}

#define enable_timer_configuration()    {timer_configuration=ENABLED;}
#define disable_timer_configuration()   {timer_configuration=DISABLED;}

int count=0;

void stopbit_interrupt(void)
{
  if(SCON1&0x2)
  {
    set_serial1_tbe_flag();
    clear_bit(SCON1,SCON1_TI);
  }

  if(SCON1&0x1)
  {
    set_serial1_rbf_flag();
    clear_bit(SCON1,SCON1_RI_MASK);
  }

  if(timer_configuration==ENABLED)
  {
    conf_timer21_etu_timeout_value();
    start_timer21();
  }
  return;
}

/*
 * Returns version string
 */
char *dssc_version(void)
{
    return (char *)DSSC_VERSION;
}

void wait_smartcard_clocks(unsigned int clock)
{
   T2CNA2=0;        // timer off
   T2CNB2=0;

   // Receive a byte or timeout within millisecond timeout
   T2CFG2=0x020;    // timer mode, div by 4, sixteen bit

   T2V2=65536-(clock);
   T2R2=65536-(clock);
   start_timer22();

   while(!read_bit(T2CNB2,T2CNB2_TF2));
   return;
}

void wait_etus(unsigned long etu)
{
   unsigned int iteration, initial_load;
    etu=((etu*372L)/D);
    iteration=(etu/0x10000)+1;

    T2CNA2=0;      // timer off
    T2CNB2=0;

    // Receive a byte or timeout within millisecond timeout
    T2CFG2=0x020;  // timer mode, div by 4, sixteen bit

    T2V2=65536-(etu%0x10000);
    T2R2=0;
    start_timer22();

    while(iteration--)
    {
      while(!read_bit(T2CNB2,T2CNB2_TF2));
      clear_bit(T2CNB2,T2CNB2_TF2);
    }

    return;
}

void conf_timer22_clock_timeout_value(unsigned int clock_cycles)
{

   T2CNA2=0;        // timer off
   T2CNB2=0;

   // Receive a byte or timeout within millisecond timeout
   T2CFG2=0x020;    // timer mode, div by 4, sixteen bit

   T2V2=65536-(clock_cycles);
   T2R2=65536-(clock_cycles);
   return;
}

void conf_timer21_clock_timeout_value(unsigned int clock_cycles)
{

   T2CNA1=0;        // timer off
   T2CNB1=0;

   // Receive a byte or timeout within millisecond timeout
   T2CFG1=0x020;    // timer mode, div by 4, sixteen bit

   T2V1=65536-(clock_cycles);
   T2R1=65536-(clock_cycles);
   return;
}


void conf_timer21_etu_timeout_value(void)
{

    T2CNA1=0;        // timer off
    T2CNB1=0;

    // Receive a byte or timeout within millisecond timeout
    T2CFG1=0x020;    // timer mode, div by 4, sixteen bit

    T2V1=65536-timer1_initial_load;
    T2R1=0;
    timer1_iteration=timer1_num_of_iteration;

    return;
}

void conf_timer22_etu_timeout_value(void)
{

    T2CNA2=0;      // timer off
    T2CNB2=0;

    // Receive a byte or timeout within millisecond timeout
    T2CFG2=0x020;  // timer mode, div by 4, sixteen bit

    T2V2=65536-timer2_initial_load;
    T2R2=0;
    timer2_iteration=timer2_num_of_iteration;
    return;
}


void calc_timer21_etu_timeout_value(unsigned long etus)
{
    etus*=372L/D;

    timer1_initial_load = etus%0x10000;
    timer1_num_of_iteration = etus/0x10000;

    return;
}

void calc_lastbyte_timer21_etu_timeout(unsigned long etus)
{
    etus=((etus*372L/D)+(120L/D));

    timer1_initial_load = etus%0x10000;
    timer1_num_of_iteration = etus/0x10000;

    return;
}


void calc_timer22_etu_timeout_value(unsigned long etus)
{ 
    etus*=372L/D;

    timer2_initial_load = etus%0x10000;
    timer2_num_of_iteration = etus/0x10000;

    return;
}


/**
 * \brief   Reads a byte from card using maxq2000 uart in T1 mode
 *
 * \return  Positive One byte data, 
 * Otherwise, returns one of the following error codes:
 *           #ERR_RECEIVE_TIMEOUT
 */
int16_t readT1Byte(void)
{
  uint8_t recv_byte, scon_status;
  
  while(1)
  {
      if(read_bit(T2CNB1, T2CNB1_TF2))
      {
          if(!timer1_iteration)
          {
              return ERR_POWERUP_ATR_TIMEOUT;
          }
          timer1_iteration--;
          clear_bit(T2CNB1,T2CNB1_TF2);
      }

      if(serial1_rbf_flag==DISABLED)
      {
          continue;
      }

      clear_serial1_rbf_flag();

      recv_byte=SBUF1;
      scon_status=SCON1;                  // Save serial status flags
          
      if(card_convention==DIRECT_CONVENTION)
      {
            if(read_bit(scon_status, SCON1_RB8)!=parity_of_direct[recv_byte])
            {
                t1_parity_error=1;
            }
            return recv_byte;
      }
      else
      {
            if(read_bit(scon_status,SCON1_RB8)!=(parity_of_direct[recv_byte]^1))
            {
                t1_parity_error=1;
            }
            return (int)inverse_of[recv_byte];
      }
  }
}

/**
 * \brief   Write a byte to card using maxq2000 uart in T1 protocol mode
 *
 * \param   onebyte  one byte value to be sent to card
 *
 */
void write_T1_Byte(uint8_t onebyte)
{
  uint8_t parity_bit;

  if(card_convention==DIRECT_CONVENTION)
  {
    parity_bit=parity_of_direct[onebyte];
  }
  else
  {
    parity_bit=(parity_of_direct[onebyte]^1);
  }

//  while(!read_bit(SCON1, SCON1_TI));
  while(serial1_tbe_flag==DISABLED);

  wait_smartcard_clocks(450/D); //wait for atleast one etu before sending another char

  if(GTR) wait_etus(GTR);     //wait for extra guard time if it is set in ATR

  clear_serial1_tbe_flag();

  if(parity_bit) 
     set_bit(SCON1,SCON1_TB8);
  else
     clear_bit(SCON1,SCON1_TB8);

  SBUF1=onebyte;

  return;
}

void writeBlock(uint8_t NAD, uint8_t PCB, uint8_t length, uint8_t *buffer)
{
    uint8_t block_data[MAX_BLOCK_LEN], *bufptr, i=0;
    uint8_t EDC=0,len;

    len=length;

    EDC = generateEDC(NAD,EDC);
    EDC = generateEDC(PCB,EDC);
    EDC = generateEDC(len,EDC);

    if(card_convention==INVERSE_CONVENTION)
    {
       NAD=inverse_of[NAD];
       PCB=inverse_of[PCB];
       len=inverse_of[length];
    }
    
    for(i=0;i<length;i++)
    {
          if(card_convention==INVERSE_CONVENTION)
          {
            block_data[i]=inverse_of[buffer[i]];
          }
          EDC = generateEDC(buffer[i],EDC);
    }

    if(card_convention==INVERSE_CONVENTION)
    {
      EDC=inverse_of[EDC];
      bufptr=block_data;
    }
    else
    {
       bufptr=buffer;
    }

    //disable serial port receive
    clear_bit(SCON1,SCON1_REN);
    disable_timer_configuration();

    write_T1_Byte(NAD);
    write_T1_Byte(PCB);
    write_T1_Byte(len);

    for(i=0;i<length;i++)
    {
       write_T1_Byte(bufptr[i]);
    }

    enable_timer_configuration();
    write_T1_Byte(EDC);

    //enable serial port receive
    clear_bit(SCON1,SCON1_RI_MASK);
    clear_serial1_rbf_flag();
    while(!serial1_tbe_flag);
    calc_timer21_etu_timeout_value(CWT);
    set_bit(SCON1,SCON1_REN);

    return;
}

uint16_t parity_signal_wait_time[5]={0,276,115,0,45};
uint16_t parity_signal_set_time[5]={0,496,248,0,124};

/**
 * \brief   Reads byte from card using maxq2000 uart in t0 mode
 * 
 * \return  Positive One byte data, 
 * Otherwise, returns one of the following error codes:
 *           #ERR_RECEIVE_TIMEOUT
 *           #ERR_RECEIVE_PARITY
 */
int16_t readT0Byte(void)
{
  uint8_t recv_byte, scon_status, parity_error_count=0;
  
  do
  {
      if(read_bit(T2CNB1, T2CNB1_TF2))
      {
          if(!timer1_iteration--)
          {
              dssc_powerdown();
              return ERR_POWERUP_ATR_TIMEOUT;
          }
          clear_bit(T2CNB1,T2CNB1_TF2);
      }

      if(serial1_rbf_flag==DISABLED)
      {
          continue;
      }
      clear_serial1_rbf_flag();

      recv_byte=SBUF1;
      scon_status=SCON1;                  // Save serial status flags

      if(card_convention==DIRECT_CONVENTION)
      {
            if(read_bit(scon_status, SCON1_RB8)!=parity_of_direct[recv_byte])
            {   
             
                 wait_smartcard_clocks(parity_signal_wait_time[D]);
                 //send parity error signal to card
                 clear_bit(SCON1, SCON1_REN);
                 clear_bit(PO5, IOUC_TX_OUT);
                 configure_out(PD5,IOUC_TX_DIR);
                 wait_smartcard_clocks(parity_signal_set_time[D]); //wait for two etus
                 set_bit(PO5, IOUC_TX_OUT);
                 configure_in(PD5,IOUC_TX_DIR);
                 set_bit(SCON1, SCON1_REN);
            }
            else
            {
                 return (int)recv_byte;
            }
      }
      else 
      {
            if(read_bit(scon_status,SCON1_RB8)!=(parity_of_direct[recv_byte]^1))
            {
                // wait_smartcard_clocks(186/D);
                 wait_smartcard_clocks(parity_signal_wait_time[D]);
                 //send parity error signal to card
                 clear_bit(SCON1, SCON1_REN);
                 clear_bit(PO5, IOUC_TX_OUT);
                 configure_out(PD5,IOUC_TX_DIR);
                 wait_smartcard_clocks(parity_signal_set_time[D]); //wait for two etus
                 set_bit(PO5, IOUC_TX_OUT);
                 configure_in(PD5,IOUC_TX_DIR);
                 set_bit(SCON1, SCON1_REN);
            }
            else
            {
                 return(int)inverse_of[recv_byte];
            }
       }
  parity_error_count++;
  }while(parity_error_count<=4);

  dssc_powerdown();
  return ERR_RECEIVE_PARITY;
}

/**
 * \brief   Write a byte to card using maxq2000 uart in T0 protocol mode
 *
 * \param   onebyte  one byte value to be sent to card
 *
 * \return  Length of APDU buffer returned, 
 * Otherwise, returns #DS8113LIB_STATUS_SUCCESS
 *                    #ERR_RECEIVE_PARITY
 *
 */
int16_t write_T0_Byte(uint8_t onebyte)
{
  uint8_t parity_error_count=0, parity_bit;

  if(card_convention==DIRECT_CONVENTION)
  {
    parity_bit=parity_of_direct[onebyte];
  }
  else
  {
    parity_bit=(parity_of_direct[onebyte]^1);
    onebyte=inverse_of[onebyte];
  }

  //disable serial port receive
  clear_bit(SCON1, SCON1_REN);
  disable_timer_configuration();

  while(serial1_tbe_flag==DISABLED);

  if(GTR) wait_etus(GTR);

  do {

      clear_serial1_tbe_flag();

      if(parity_bit) 
        set_bit(SCON1,SCON1_TB8);
      else
        clear_bit(SCON1,SCON1_TB8);

      SBUF1=onebyte;

      while(!serial1_tbe_flag);
      
      //wait for around 1 etus before sampling error signal
      wait_smartcard_clocks(372/D); //around 1 etus

      if(read_bit(PI5,IOUC_RX_IN)==1)
      {
          wait_smartcard_clocks(372/D); //wait for two etus
          return DS8113LIB_STATUS_SUCCESS;
      }

      //wait for 2 etus before repeating byte
       wait_smartcard_clocks(744/D); //wait for two etus

      } while(parity_error_count++<4);

    dssc_powerdown();
    return ERR_RECEIVE_PARITY;
}

/**
 * \brief  Write a byte and configure the DS8113 in receive mode after transmitting byte in T0 protocol mode
 *
 * \param   onebyte One byte value to send to card
 * \return  #DS8113LIB_STATUS_SUCCESS is returned for success, 
 * Otherwise, returns 
 *          #ERR_RECEIVE_PARITY
 */
int16_t write_T0_LastByte(uint8_t onebyte)
{
  uint8_t parity_error_count=0, parity_bit;

  if(card_convention==DIRECT_CONVENTION)
  {
    parity_bit=parity_of_direct[onebyte];
  }
  else
  {
    parity_bit=(parity_of_direct[onebyte]^1);
    onebyte=inverse_of[onebyte];
  }

  //disable serial port receive
  clear_bit(SCON1,SCON1_REN);
  clear_serial1_rbf_flag();
  calc_lastbyte_timer21_etu_timeout(WWT);
  enable_timer_configuration();

  while(serial1_tbe_flag==DISABLED);


  if(GTR) wait_etus(GTR);


  do {

      clear_bit(SCON1,SCON1_TI);
      clear_serial1_tbe_flag();

      if(parity_bit) 
        set_bit(SCON1,SCON1_TB8);
      else
        clear_bit(SCON1,SCON1_TB8);

      SBUF1=onebyte;

      while(!serial1_tbe_flag);

      //wait for around 1 etus before sampling error signal
      wait_smartcard_clocks(372/D); //around 1 etus

      if(read_bit(PI5,IOUC_RX_IN)==1)
      {
          //enable serial port receive
          set_bit(SCON1,SCON1_REN);
          clear_serial1_rbf_flag();
          clear_bit(SCON1,SCON1_RI_MASK);
          calc_timer21_etu_timeout_value(WWT);
          return DS8113LIB_STATUS_SUCCESS;
      }

      //wait for 2 etus before repeating byte
       wait_smartcard_clocks(744/D); //wait for 2 etus

      } while(parity_error_count++<4);
    dssc_powerdown();
    return ERR_RECEIVE_PARITY;
}
   
/**
 * \brief   Send a T=1 formatted block. Formats T=1 packet from raw input data.
 *  Length must be within IFSC value.
 * 
 * \param   NAD 
 * \param   PCB 
 * \param   length 
 *
 */
void sendBlock(uint8_t NAD, uint8_t PCB, uint8_t length, uint8_t *buffer)
{
  uint8_t EDC;
  uint32_t timeout;

  //Calculate Wait Time Extension or Block Waiting time while receiving data
  //after sending last byte
  if(wtx>0)
  {
      timeout = (1L* BWT*wtx);  // Use BWT for T=1
  }
  else
  {
      timeout = BWT;  // Use BWT for T=1
  }

  calc_lastbyte_timer21_etu_timeout(timeout);
  set_serial1_tbe_flag();
  clear_bit(SCON1,SCON1_TI_MASK);

  //Make sure we follow BGT timing
  //Follow BGT time before sending data at opposite direction
  wait_etus(13);//we are waiting 4 more etus to be in safer side

  //send out the block
  writeBlock(NAD,PCB,length,buffer);
  
  return;
}

/**
 * \brief   Receive a T=1 formatted block. This does a raw receive, 
 * it does not check sequence numbers.
 * 
 * \param   rNAD
 * \param   rPCB
 * \param   rLEN
 * \return  
 * Otherwise, returns one of the following error codes:
 *           #ERR_RECEIVE_TIMEOUT
 *           #ERR_RECEIVE_PARITY
 *           #ERR_ABORT_T1_TRANSACTION
 *           #ERR_MAX_RETRIED 
 *
 */
int16_t receiveBlock(uint8_t *rNAD, uint8_t *rPCB, uint8_t *rLEN)
{
  int16_t retval;
  int16_t index = 0;
  uint8_t EDC;
  int16_t i;
  uint8_t *buffer;

  //Get working buffer of current cardslot.
  buffer=card_workingBuffer;

  t1_parity_error=0;

  // Get NAD, PCB, and Length
  retval = readT1Byte();
  if (retval < 0)  { goto exit_from_receiveblock;}
  *rNAD = retval;
  
  retval = readT1Byte();
  if (retval < 0)  { goto exit_from_receiveblock;}
  *rPCB = retval;

  retval = readT1Byte();
  if (retval < 0)  { goto exit_from_receiveblock;}
  *rLEN = retval;

  // Add additional byte if using CRC
  // Get all data bytes plus EDC (1 byte at end)
  for (i = 0;i <= *rLEN;i++)
  {
    retval = readT1Byte();
    if (retval < 0)
    {
       goto exit_from_receiveblock; //if there is timeout, abort LRC, Parity Checking
    }                                     
    buffer[index++] = retval;
  }

  //compute LRC of received block
  EDC = 0;
  EDC = generateEDC(*rNAD,EDC);
  EDC = generateEDC(*rPCB,EDC);
  EDC = generateEDC(*rLEN,EDC);
  for (i = 0;i <= *rLEN;i++)
  {
      EDC = generateEDC(buffer[i],EDC);
  }

  //check if there is any parity error  
  if(t1_parity_error)
  {
      retval=ERR_RECEIVE_PARITY;
      goto exit_from_receiveblock;
  }

  //Check if there is any LRC error
  if (EDC != 0)
  {
      retval=ERR_RECEIVE_LRC;
      goto exit_from_receiveblock;
  }

  //Check the received T1 block format
  if (*rNAD != 0)
  {
      retval=ERR_INVALID_NAD;
      goto exit_from_receiveblock;
  }

  if((ISRBLOCK(*rPCB)&&(*rLEN!=0))||(ISSBLOCK(*rPCB)&&(*rLEN>1)))
  {
      retval=ERR_INVALID_LEN;
  }
  else
  {
      retval=*rLEN;
  }

exit_from_receiveblock:

  //if length is FF, abort APDU transaction
  if(*rLEN==0xFF)
  {
     return ERR_ABORT_T1_TRANSACTION;
  }

  return retval;
}

/********************************************************************
 *  See lib8113.h header file for function description              *
 *******************************************************************/
int16_t dssc_sendsblockIFSD(uint8_t IFSD)
{
  uint8_t buffer[1];
  uint8_t rNAD,rPCB,rLEN;
  uint8_t *rbuffer;
  uint8_t count=T1_BLOCK_RETRY;  

  buffer[0] = IFSD;     //IFSD value should always be 0xFE as per EMV4.1

  //Get working buffer of current cardslot.
  rbuffer=card_workingBuffer;

  do
  {
      sendBlock(0,0xC1,1,buffer); //Send IFSD request to card
  
      if((receiveBlock(&rNAD,&rPCB,&rLEN))<0) //Error in received block, retry for 3 times
      {
          continue;
      }

      // If we do not get a confirmation response, fail.
      if ((rPCB == 0xE1) && (rLEN == 1) && (rbuffer[0] == IFSD))
      {
          return 0;
      }

  }while(--count);

  return ERR_SETIFSD_FAILURE;
}

/*
Set Node address info for this card
*/
void dssc_setNAD(uint8_t value)
{
  NAD = value;
}

/**
 * \brief  Receives T1 protocol block from card and processes it if it is S-block
 * 
 * \param   rNAD
 * \param   rPCB
 * \param   rLEN
 * \return  Length of APDU buffer returned, 
 * Otherwise, returns one of the following error codes:
 *           #ERR_RECEIVE_TIMEOUT
 *           #ERR_RECEIVE_PARITY
 *           #ERR_INVALID_STATUS_BYTE
 *           #ERR_UNKNOWN_ERROR
 *           #ERR_ABORT_T1_TRANSACTION
 *           #ERR_MAX_RETRIED 
 *
 */
int receiveSBlock(uint8_t *rNAD, uint8_t *rPCB, uint8_t *rLEN)
{
  int ret; 
  int Responsed;

  //Get working buffer of current cardslot.
  uint8_t *rbuffer=card_workingBuffer;

  t1_sblock_sent=0;     

  do
  {
    Responsed = 0;

    //if there is parity, lrc or timeout error, return status
    //to sendAPDUt1 function to handle it.
    if((ret = receiveBlock(rNAD,rPCB,rLEN))<0)
    {
        return ret;
    }

    switch (*rPCB)
    {
      case SREQUEST_ABORT :                     //Card requests to ABORT, send ABORT error code
      {                                         //to abort transaction
           return ERR_ABORT_T1_TRANSACTION;
      }

      case SREQUEST_IFS :                       //Card sends IFSC request
        if((*rbuffer<0x10)||(*rbuffer==0xff))   //Check if it is valid IFSC request
        {
           return ERR_INVALID_INF;
        }

        sendBlock(0,SRESPONSE_IFS,1,rbuffer);   //Send IFSC response
        IFSC = *rbuffer;                        //Store new IFSC value in buffer
        Responsed = 1;                          //Set t1 status flags to maintain protocol state
        t1_sblock_sent=1;
        break;  

      case SREQUEST_WTX :                       //Card sends WTX Request
         wtx=*rbuffer;                          //Store new WTX value in wtx buffer
         sendBlock(0,SRESPONSE_WTX,1,rbuffer);  //Send wtx response to card
         wtx=0;                                 //Previous sendblock used wtx timing for receiving block,
         Responsed = 1;                         //So, Reset wtx buffer
         t1_sblock_sent=1;                      //Set t1 status flags to maintain protocol state
         break;
    }

  } while (Responsed);  
  return ret;
}

/*****************************************************************************
 *  See lib8113.h header file for function description                       *
 *****************************************************************************/
int16_t dssc_sendAPDUT1(uint8_t *req_buf, uint16_t req_len, uint8_t *resp_buf)
{
  int ret;
  int currentIBlockLen, more, retry, i_block_retry;
  int totalResLength;
  uint8_t rNAD, rblock_sent=0;
  int cardiblock_err=0;
  uint8_t rPCB;
  uint8_t rLEN;
  uint8_t *IFDSeq, *ICCSeq;

  // Get working buffer of current cardslot.
  uint8_t *rbuffer=card_workingBuffer;

  IFDSeq=&IFD_Seq;        // Initialize IFDSeq and ICCSeq numbers
  ICCSeq=&ICC_Seq;

  // Make sure IFSD Negotiation takes place once before any APDU exchange for T1 protocol as per EMV4.1
  if (IFSD_Negotiated==0)
  {
    if((ret=dssc_sendsblockIFSD(0xFE))<0)
    {
        return ret;
    }
    IFSD_Negotiated = 1;
  }

  //Send APDU request
  while (req_len) 
  {
    currentIBlockLen = (req_len<IFSC)?req_len:IFSC; 
  
    //We do chaining if request length is greater than IFSC
    req_len -= currentIBlockLen;
    more = req_len ? 1 : 0;     
    
    retry = T1_BLOCK_RETRY;

    i_block_retry=1;
    cardiblock_err=0;

    sendBlock(0,CODEIBLOCK(*IFDSeq,more), currentIBlockLen,req_buf);
    
    //make sure block gets sent successfully within three times
    while (retry--)
    {
      ret = receiveSBlock(&rNAD,&rPCB,&rLEN); //receive response from card for I-block

      if ((ret >= 0 ))
      {

          if((ISRBLOCK(rPCB)))    //Check whether card sent R-Block
          {
                //Check IFDseq to see whether wants to re-transmit previous block
                if((NR(rPCB) == *IFDSeq)&&(!t1_sblock_sent))
                {  
                    if(((++i_block_retry)>3))
                    {
                        retry=-1;
                        ret=ERR_MAX_RETRIED;
                        break;
                    }

                    sendBlock(0,CODEIBLOCK(*IFDSeq,more), currentIBlockLen, req_buf);
                    retry++;  
                    continue;
                 }
                 else 
                 { 
                    if (more) //Card sent ACK for previous i-block, send next chained i-block
                    {
                        *IFDSeq = 1 - *IFDSeq; 
                        break;  
                    }
                    else if(((!rblock_sent)&&(retry<2))) //there could be error in icc response to last i-block
                    {
                        sendBlock(0,CODERBLOCK(*ICCSeq, ((cardiblock_err==ERR_RECEIVE_LRC)||(cardiblock_err==ERR_RECEIVE_PARITY)) ? 1 : 2), 0,(void*)0);
                        retry++;
                        rblock_sent=1;
                        continue;
                    }
                 }
            }
            else if (ISIBLOCK(rPCB)) //Card started sending APDU response, getinto receive APDU state machine
            { 
                if (((NS(rPCB) == *ICCSeq) && (more==0)))
                {
                    *IFDSeq = 1 - *IFDSeq;
                    break; 
                  }
               }
          } 

       //Card wants to abort T1 transaction, return with Abort Error Code.
       if(ret== ERR_ABORT_T1_TRANSACTION)
       {
            return ERR_ABORT_T1_TRANSACTION;
       }

       if(retry)
       {
            if((cardiblock_err))
            {
               ret=cardiblock_err;
            }
            
            //send R block saying we got invalid block
            sendBlock(0,CODERBLOCK(*ICCSeq, ((ret==ERR_RECEIVE_LRC)||(ret==ERR_RECEIVE_PARITY)) ? 1 : 2), 0,(void*)0);
            
            cardiblock_err =ret;
       }
    }

    //The transaction errors exceeded EMV limit, so, return error code
    if (retry < 0)
    {
        return ERR_MAX_RETRIED;    
    }
    req_buf += currentIBlockLen;
  }

  totalResLength = 0;

  do
  {
    memcpy(resp_buf,rbuffer, rLEN);
    resp_buf += rLEN;
    totalResLength += rLEN;
    
    *ICCSeq = 1 - *ICCSeq;

    //Check whether there are more chained APDU responses blocks
    if (MORE(rPCB))
    {
      sendBlock(0,CODERBLOCK(*ICCSeq, 0), 0, (void*)0); //Send ACK for received I-block
      retry = T1_BLOCK_RETRY;

      while (retry)
      {
        ret = receiveSBlock(&rNAD,&rPCB,&rLEN); //Receive Next Block from Card

        if(ret >= 0)
        {
          if (ISRBLOCK(rPCB))  //Card did not receive our ACK properly, resend it
          {
             if (!(NR(rPCB) != *IFDSeq))
             {
                 /* resent our R-block */
                sendBlock(0,CODERBLOCK(*ICCSeq, 0), 0,(void*)0);
                continue;  
             }
          }
          else if (ISIBLOCK(rPCB))
          {
              if (NS(rPCB) == *ICCSeq)
              {
                break;  /* A valid I-Block,break this loop */
              }    
          }
        }

        if(ret== ERR_ABORT_T1_TRANSACTION)    //Card wants to ABORT Transaction, return with Error Code
        {
            return ERR_ABORT_T1_TRANSACTION;
        }

        if(--retry)
        {
              //send R block saying we got invalid block
            sendBlock(0,CODERBLOCK(*ICCSeq, 0), 0,(void*)0);
        }
      }

      if (!retry)
      {
           return ERR_MAX_RETRIED;
      }
    }
    else
    {
        break;
    }
  } while (1);
  return totalResLength;
}

/*****************************************************************************
 *  See lib8113.h header file for function description                       *
 *****************************************************************************/
int16_t dssc_sendAPDUT0(uint8_t *buffer, uint16_t length, uint8_t *rbuffer)
{
  int index;
  uint16_t rindex = 0, Lc,Le;
  uint8_t val;
  uint8_t INS,P3=0,wait_for_proc_byte,*hdr, SW1, SW2, warning_status_byte=0,case4_cmd=0,get_resp=0,inverse_INS;
  int retval;

  uint8_t wlt1,wlt2,wlt3, rt1,rt2,rt3;

  Lc=Le=0;

  set_serial1_tbe_flag();
  clear_bit(SCON1,SCON1_TI_MASK);

  //Identify the type of APDU transaction  
  if(length>5)
  {
    Lc=buffer[4];
    if(length>Lc+5)
    {   //check whether it is case3 command or case4 command
          Le=buffer[length-1];
          case4_cmd=1;
    }
    P3=Lc;
  }
  else if(length==5)    //check whether it is case2 command or case1 command
  {
    Le=buffer[4];
    P3=Le;
  }

  hdr=buffer;

  //Follow BGT time before sending data at opposite direction
  wait_etus(6); ////we are waiting 2 more etus to be in safer side


  //state machine to process t=0 apdu request and response
  while(1)
  {    

    INS=hdr[1];
    inverse_INS=~INS;
    wait_for_proc_byte=1;

    //disable serial port receive
    clear_bit(SCON1,SCON1_REN);

    // Write 5 byte command header
    for (index = 0;index < 4;)
    {
        retval=write_T0_Byte(hdr[index++]);
        if (retval < 0)
        {
              return retval;
        }
    }

    //set work wait timeout after writing last byte
    retval=write_T0_LastByte(P3);

    if (retval < 0)
    {
         return retval;
    }
    
    index=5;

    //We are expecting procedure byte from Card
    while (wait_for_proc_byte)
    {
    
        // Get procedure byte
        retval = readT0Byte();

        if (retval < 0)    
        {
              return retval;
        }

        val=retval;

        //interpret procedure byte
        switch(val)
        {                                    //Card wants us to send GETresponse to get APDU response
            case 0x61:
                      get_resp=1;
            case 0x6C:                        //Card wants to repeat APDU header with expect APDU response byte
                      Lc=0;
                      hdr=(val==0x61)?GET_RESPONSE:buffer;
                      P3=Le=retval=readT0Byte();
                      Le+=rindex;
                      if(retval<0) return retval;
                      wait_for_proc_byte=0;
                      //wait at least four etus. 
                      //character wait time between opposite direction should be atleast 16 etus.
                      wait_etus(6); //we are waiting 2 more etus to be in safer side

            case 0x60:                       //Card wants to have additional WWT
                      break;
            default:
                if((val==INS)||(val==inverse_INS))  //Card wants to proceed data transaction,~INS means only one byte
                 {
                   if((Lc+5)==index)    //if we are at receive mode, receive APDU response from card
                   {
                      while((Le>rindex) && ((retval = readT0Byte()) >= 0))
                      {
                         rbuffer[rindex++] = retval;
                       if(val==inverse_INS)
                         {
                           break;
                         }
                      }

                      if (retval < 0)
                        return retval;
                       break;                      
                   }
                    
                    //We are at transmit mode, wait at least four etus.
                    //Follow BGT time before sending data at opposite direction
                    wait_etus(6); //we are waiting 2 more etus to be in safer side
 
                    //if we received INS byte, send all remaining bytes, otherwise, send only one byte 
                    if(val==INS)
                    {
                          for (;index < (Lc+4);index++)
                          {
                            retval = write_T0_Byte(buffer[index]);
                      
                            if (retval < 0)
                            {
                              return retval;
                            }
                          }
                    }

                  retval=write_T0_LastByte(buffer[index++]);
      
                  if (retval < 0)
                  {
                      return retval;
                  }
                  break;
              }

              //Card send Status bytes, check status byte values whether it is as per EMV4.1
              if(((val&0xf0)!=0x60)&&((val&0xf0)!=0x90))
              {
                  dssc_powerdown();
                  return ERR_INVALID_STATUS_BYTE;
              }

              retval=readT0Byte();
              if (retval < 0) return retval;

              //If card did not send warning status bytes earlier, then,
              //process status bytes
              if(!warning_status_byte)
              {
                  SW2=retval;    
                  SW1=val;
                  //Check status byte to see whether card has more data to read.
                  if((((SW1==0x62)||(SW1==0x63))||(((SW1&0x90)==0x90)&&(!((SW1==0x90)&&(SW2==0)))))&&((Lc+5)==index)&&(case4_cmd)&&(get_resp==0))
                  {
                      warning_status_byte=1;

                      Lc=0;
                      hdr=buffer=GET_RESPONSE;
                      P3=Le=0;
                      Le+=rindex;
                      wait_for_proc_byte=0;

                      //Follow BGT time before sending data at opposite direction
                      wait_etus(6);//we are waiting 2 more etus to be in safer side
                  }
                  else      //Card completed APDU response, send response with Status code
                  {
                      rbuffer[rindex++]=SW1;
                      rbuffer[rindex++]=SW2;
                      return rindex;
                  }     
              }
              else
              {
                      if(rindex!=0)     //If card did not send any response, send first warning status code to application
                      {
                          rbuffer[rindex++]=SW1;
                          rbuffer[rindex++]=SW2;
                      }
                      else   //Send APDU response with latest warning status code
                      {
                            rbuffer[rindex++]=val;
                            rbuffer[rindex++]=retval;
                      }
                      return rindex;
              }
          }
      }
  }
}

/*****************************************************************************
 *  See lib8113.h header file for function description                       *
 *****************************************************************************/
int16_t dssc_sendAPDU(uint8_t *buffer,uint16_t length,uint8_t *rbuffer)
{
  int ret;

  enable_stopbit_interrupt();

  switch (TMode) //Check current active protocol type
  {
    case 0:             // It is T=0, perform T0 APDU Transaction
      ret = dssc_sendAPDUT0(buffer,length,rbuffer); 
      disable_stopbit_interrupt();
      return ret;
      break;
    case 1:             // It is T=1, perform T1 APDU Transaction
      if((ret=dssc_sendAPDUT1(buffer,length,rbuffer))<0)
      {
       dssc_powerdown();
      }
      disable_stopbit_interrupt();
      return ret;
      
      break;
    default:           // We dont support other protocols currently
      return ERR_PROTOCOL_UNSUPPORTED;
      break;
  }
}

void initialize_smartcard_uart(void)
{
   //Set mode 3, async 11 bit, use baud clock
   //receive disable
   SCON1=0x0C0;

   // Turn on framing detect, /64
   SMD1=0x05;

   //Set phase register
   PR1=PHASE_SMARTCARD;

   set_bit(SCON1,SCON1_REN); //receive enable
 
   start_clock();

   return;
}

int dssc_read_first_atr_byte(void)
{
  uint8_t recv_byte, scon_status,counter;
  
   while(serial1_rbf_flag==DISABLED)
   {
       if(read_bit(T2CNB1, T2CNB1_TF2))
       {
          return ERR_POWERUP_ATR_TIMEOUT;
       }
   }

   start_timer22();

   //INFO: we are not going to bother abt framing error.
   //it seems that EMV does not care abt it.

   recv_byte=SBUF1;
   clear_bit(SCON1, SCON1_RI_MASK);
   clear_serial1_rbf_flag();

   scon_status=SCON1;                  // Save serial status flags
          
   if((recv_byte==0x3b)&&(scon_status&SCON1_RB8))
   {
      card_convention=DIRECT_CONVENTION;
   }
   else if((recv_byte==0x3)&&(scon_status&SCON1_RB8))
   {
      card_convention=INVERSE_CONVENTION;
      recv_byte=inverse_of[recv_byte];
   }
   else
     return ERR_POWERUP_ATR_INVALID;

   return (int)recv_byte;
}

int dssc_read_atr_byte(void)
{
  uint8_t recv_byte, scon_status,counter;
  
  while(serial1_rbf_flag==DISABLED)
  {
            if(read_bit(T2CNB1, T2CNB1_TF2))
            {
               if(!timer1_iteration--)
               {
                  return ERR_POWERUP_ATR_TIMEOUT;
               }
               clear_bit(T2CNB1,T2CNB1_TF2);
            }

            if(read_bit(T2CNB2, T2CNB2_TF2))
            {
               if(!timer2_iteration--)
               {
                  return ERR_POWERUP_ATR_TIMEOUT;
               }
               clear_bit(T2CNB2,T2CNB2_TF2);
            }
  }


  recv_byte=SBUF1;
  clear_bit(SCON1,SCON1_RI_MASK);
  clear_serial1_rbf_flag();

  scon_status=SCON1;                  // Save serial status flags

  if(card_convention==DIRECT_CONVENTION)
  {
     if(read_bit(scon_status,SCON1_RB8)!=parity_of_direct[recv_byte])
     {
        return ERR_RECEIVE_PARITY;
     }
  }
  else
  {
     if(read_bit(scon_status,SCON1_RB8)!=(parity_of_direct[recv_byte]^1))
     {
           return ERR_RECEIVE_PARITY;
     }
     else
     {
           recv_byte=inverse_of[recv_byte];
     }
  }

  return (int)recv_byte;
}

/*****************************************************************************
 *  See lib8113.h header file for function description                       *
 *****************************************************************************/
int16_t dssc_warmreset(void)
{
  uint8_t val;
  uint16_t status;

  configure_in(PD6, CMDVCC_DIR);

  // Check for power status
  if(read_bit(PI6, CMDVCC_OUT))
  {
    return ERR_POWERUP_VOLTAGE_INVALID;
  }

  // Initialize smartcard uart
  initialize_smartcard_uart();

  // Apply reset
  clear_bit(PO6, RSTIN_OUT);
  configure_out(PD6, RSTIN_DIR);

  // Call common getATR routine
  status=dssc_ATRsequence(WARM_RESET);
  clear_bit(SCON1, SCON1_REN);
  disable_stopbit_interrupt();
  wait_etus(2); // Wait for couple of etus. 

  return status;
}

/*****************************************************************************
;*
;* Function Name: start_clock
;*
;*   Description: Initialize timer 0 to generate something in the 3-5MHz range.
;*
;*      Input(s): None.
;*
;*     Output(s): None.
;*
;****************************************************************************
*/
void start_clock(void)
{
   T2CFG0=0;  // timer mode, div by 1, sixteen bit
   T2CNA0=0;  // timer off
   T2CNB0=0;

   T2V0 = 65536-SMARTCARD_CLK_RELOAD;
   T2R0 = 65536-SMARTCARD_CLK_RELOAD;

   T2CNA0=0x048;    // Start timer, output pin enabled
   return;
}

/*****************************************************************************
 *  See lib8113.h header file for function description                       *
 *****************************************************************************/
int16_t dssc_powerup(uint8_t voltage)
{
 uint16_t status;

  set_bit(PO6,RSTIN_OUT);
  configure_out(PD6,RSTIN_DIR);
   
  switch(voltage)
  {
    case POWERUP_5V:
        // Set to non-1.8V mode
        clear_bit(PO6, POWER_18V_PORADJ_OUT);
    	configure_out(PD6, POWER_18V_PORADJ_DIR);
        // Set to 5V power
	set_bit(PO3, POWER_5V3V_OUT);
	configure_out(PO3, POWER_5V3V_DIR);
      break;

    case POWERUP_3V:
       //FIXME: Add code if customer wants
      break;

    case POWERUP_1p8V:
       //FIXME: Add code if customer wants
      break;

    default:
      return ERR_POWERUP_VOLTAGE_INVALID;
      break;
  }

  // Put weak pullup on UART IO pin
  configure_in(PD5, IOUC_TX_DIR);
  set_bit(PO5, IOUC_TX_OUT);

  // Set to Div by 1 mode
  set_bit(PO3, CLKDIV1_OUT);
  configure_out(PD3, CLKDIV1_DIR);
  clear_bit(PO3, CLKDIV2_OUT);
  configure_out(PD3, CLKDIV2_DIR);

  initialize_smartcard_uart();

  // Hold card in reset
  set_bit(PO6,RSTIN_OUT);
  configure_out(PD6,RSTIN_DIR);

  configure_out(PD6, RSTIN_DIR);
  configure_out(PD6, CMDVCC_DIR);

  // power up the card
  clear_bit(PO6,RSTIN_OUT);
  clear_bit(PO6,CMDVCC_OUT);

  // Call common getATR routine
  status=dssc_ATRsequence(COLD_RESET);
  clear_bit(SCON1,SCON1_REN);
  disable_stopbit_interrupt();
  wait_etus(2); //wait for couple of etus. 
  return status;

}

/*
    Receives ATR from Card
*/
int16_t dssc_ATRsequence(uint8_t reset_type)
{
  int16_t val;
  uint8_t  index = 0;

  int16_t  i;
  uint8_t  historicalBytes;
  uint8_t  expectedCharacters, t0_only_atr;
  uint8_t  interfaceIteration = 1;
  uint8_t  done = 0;
  uint8_t  check=0;
  uint8_t  reject_atr=0;
  uint16_t TB3=-1, status=0;
  uint8_t  TCK, TA1=0x0, TC1=0, TC2=0x0A, TB1=0;

  // Default to T=0 mode
  uint8_t T = 0;
  uint8_t curByte;

  //intialize default protocol configuration
  NAD  = 0;
  IFSC = 0x20;  
  CWT = 11 + (1 << 13);
  BWT = 11L + (1L << 4L) * 960L;
  ATRmode = NEGOTIABLE_MODE;
  t0_only_atr=1;
  GTR=0; //Initialize Guard time to zero.
  D=1;

  clear_serial1_rbf_flag();
  set_serial1_tbe_flag();
  enable_stopbit_interrupt();
  enable_timer_configuration();

  // Wait 40000 to 45000 cycles to release reset
  wait_smartcard_clocks(40000);

  // power up EMV
  // Wait up to 42000 cycles for ATR to start
  //FIXME: Needs to be fine tune for testlab timing requirement
  conf_timer21_clock_timeout_value(45650);

  calc_timer22_etu_timeout_value(20150);
  conf_timer22_etu_timeout_value();

  // Reset etucount to calculate for 10080 etu between characters in ATR
  calc_timer21_etu_timeout_value(10080);

  //FIXME: Do we need to clear serial port buffer once again?
  clear_bit(SCON1, SCON1_RI_MASK);

  // Release card from reset
  set_bit(PO6,RSTIN_OUT);
  start_timer21();
  val = dssc_read_first_atr_byte();

  if (val < DS8113LIB_STATUS_SUCCESS) // Success = 0
  {
        dssc_powerdown();
        return val; 
  }

  curByte = val;
  if ((curByte != 0x3f) && (curByte != 0x3b)) // We received invalid ATR first byte
  {                                           // Return Error
      dssc_powerdown();
      return ERR_POWERUP_ATR_INVALID;
  }

  card_workingBuffer[index++] = curByte;

  while (!done)
  {

    val = dssc_read_atr_byte();

    if (val < DS8113LIB_STATUS_SUCCESS)
    { 
        dssc_powerdown();
        return val; 
    }

    curByte = val;
    card_workingBuffer[index++] = val;

    check ^= curByte;                                // Start calculating checksum

    if (index == 2)                                  // We received T0 byte
    {
        historicalBytes = curByte & 0x0F;            // Find out historical bytes
        expectedCharacters = curByte & 0xF0;         // Figure out upcoming protocol conf bytes
    }                                              

    if (index > 2)
    {
          switch(expectedCharacters)          // Check expected protocol conf bytes status
          {
            case 0x00:
              // Historical characters
              historicalBytes--;
              if (historicalBytes == 0)
              {
                if(t0_only_atr)
                {
                  done = 1;
                }
                else
                  expectedCharacters = 0x01;  // Go to checksum state
              }
              break;

            case 0x01:
              // TCK case
              TCK = curByte;
              done = 1;
              break;

            case 0x10:
            case 0x30:
            case 0x50:
            case 0x70:
            case 0x90:
            case 0xB0:
            case 0xD0:
            case 0xF0:
              // TA case
              expectedCharacters &= 0xE0;
              switch(interfaceIteration)
              {
                  //TA1 case
                  case 1:
                        if((curByte<0x11)||(curByte>0x13)) // Reject if we receive invalid TA1
                        {
                            reject_atr=1;
                        }
                        TA1=curByte;
                        D=TA1&0xf;
                        D=(D==3)?4:D;     // F will always be 1. Calculate D value,
                        break;            // We need to configure Phase register according to D value
                case 2:
                        if((curByte&0xf)!=T)     // If protocol is not same as 
                        {                        // first indidicated protocol, reject ATR
                            reject_atr=1;
                        }
                        else
                        {
                            // Interpret 5th bit to see whether we need to follow card specified
                            // D value or default D value
                            if(curByte&0x10)
                            {
                                reject_atr=1;
                            }
                            else
                                ATRmode=SPECIFIC_MODE_USE_TA1_F_D;
                        }
                        break;
                case 3:
                        // If IFSC value is not as per EMV4.1, reject ATR
                        if(((curByte>=0x00) && (curByte<0x10))||(curByte==0xff))
                        {
                            reject_atr=1;
                        }
                        IFSC=curByte;
              }
              break;

            case 0x20:
            case 0x60:
            case 0xA0:
            case 0xE0:
              // TB case
              expectedCharacters &= 0xD0;

              switch(interfaceIteration)
              {
                      case 1:
                        // TB1 should not be nonzero for COLDRESET as per EMV4.1
                        if((reset_type==COLD_RESET)&&(curByte!=0))
                        {
                            reject_atr=1;
                        }
                        TB1=1;
                        break;
                    case 2:        // TB2 should not be there in ATR as per EMV4.1
                            reject_atr=1;
                        break;
              }

              if(interfaceIteration==3)
              {
                TB3=curByte;                              
              }
              break;

            case 0x40:
            case 0xC0:
              // TC case
              expectedCharacters &= 0xB0;
              switch(interfaceIteration)
              {
                  case 1: 
                        TC1=curByte;
                        if((reset_type==COLD_RESET)&&(!TB1))
                        {
                            reject_atr=1;
                        }
                        break;
                case 2:
                        // Reject ATR if work wait time is not 0xA
                        if(curByte!=0x0A)
                        {
                            reject_atr=1;
                        } 
                        else
                        {
                            TC2=curByte;
                        }
                        break;
                case 3:
                        if(curByte!=0x00)
                        {
                            reject_atr=1;
                        } 
              }
              break;

            case 0x80:
              // TD case
              expectedCharacters=(curByte&0xF0);
              // Handle zero historical characters
              // If we get TD1, we have the first protocol selection
              if ((interfaceIteration==1))
              {
                  T=curByte & 0x0F;
                  if((curByte&0xF)>1)
                  {
                   reject_atr=1;
                  }
                  if(((reset_type==COLD_RESET)&&(!TB1)))
                  {
                    reject_atr=1;
                  }
              }
              else if(interfaceIteration==2)
              { 
                  // Check whether TD2 is as per EMV4.1
                  if((((curByte&0x0F)|T)!=0xE)&&((curByte&0x0F)!=1))  
                  {
                    reject_atr=1;
                  }
              }

              if((curByte&0x0F)>0)
              {
                   t0_only_atr=0;
              }
              interfaceIteration++;
              break;

            default:
              reject_atr=1;
            break;
          }

          // If we have received all atr bytes, comeout from loop
          if ((expectedCharacters == 0x00) && (historicalBytes == 0))
          {
                if (t0_only_atr)
                {
                  done = 1;
                }
                else
                  expectedCharacters = 0x01;
           }
       }
  }

  // If we are in T=1 protocol, we get a LRC (xor checksum) at the end
  // If checksum is not correct, powerdown the card
  if (T==1)
  {
    if ((check != 0)&&(reject_atr==0))
    {
      dssc_powerdown();
      return ERR_POWERUP_ATR_CRC_FAILURE;
    }
  }

  if(T==1)
  {
        i=TC1;
        i=(TC1==0xff)?-1:TC1;
        if((TB3==-1)||((TB3&0x0F)>0x05)||((TB3&0xF0)>0x40)||((1 << (TB3&0x0f))<=(i+1)))
        {
            reject_atr=1;
        }
  }

  // If ATR is rejected, try to do Warm Reset.
  if(reject_atr==1)
  {
      if(reset_type==WARM_RESET)
      {
           dssc_powerdown();
           return ERR_POWERUP_ATR_INVALID;;
      }
      else
      {
        return ERR_DO_WARM_RESET;
      }
  }
  
  // Set ATR length for the ATR received
  ATRLength = index;

  // Set T protocol mode
  TMode = T;

  // Set extra guard time
  GTR=TC1;
  if(GTR==0xff) GTR=0;

  // Set new Baudrate
  if(ATRmode==SPECIFIC_MODE_USE_TA1_F_D)
  {
      // Set new baudrate for smartcard serial uart
      PR1=(PHASE_SMARTCARD*D);
  }
  else
  {
      D=1;
  }

  // Set WWT 
  WWT = TC2 * 960L *D;
  WWT+=480L*D;
 
  // Set CWT and BWT if present in TB3
  if (TB3 != -1)
  {
      val = TB3 & 0x0F;
      CWT = 11 + (1 << val);
      val = (TB3 & 0xF0) >> 4;
      BWT = 11L + (1L << val) * 960L * D;
  }

  if(T==1)
  {
        BWT+=960L*D;
        CWT+=4;

        // Initialize T=1 protocol variables
        IFD_Seq = ICC_Seq = IFSD_Negotiated = 0;
  }

  return DS8113LIB_STATUS_SUCCESS;
}

/*****************************************************************************
 *  See lib8113.h header file for function description                       *
 *****************************************************************************/
int16_t dssc_powerdown(void)
{
  wait_etus(2);

  // Shut down the card
  set_bit(PO6, RSTIN_OUT);
  set_bit(PO6, CMDVCC_OUT);

  configure_out(PD6, CMDVCC_DIR);
  configure_out(PD6, RSTIN_DIR);

  // Power down the card
  return DS8113LIB_STATUS_SUCCESS;
}

/*****************************************************************************
 *  See lib8113.h header file for function description                       *
 *****************************************************************************/
int16_t dssc_checkpresence(void)
{
  uint8_t off_bit,cmd_bit;

  configure_in(PD6, CMDVCC_DIR);
  configure_in(PD6, OFF_DIR);
  
  //if((read_bit(PI6, OFF_IN)==1) && (read_bit(PI6, CMDVCC_IN)==1))
  off_bit = read_bit(PI6, OFF_IN);
  cmd_bit = read_bit(PI6, CMDVCC_IN);
  // Note: cmd_bit cannot be used in checkpresence as it is low before ATR
  // and off after ATR. It was removed from the test.
  //if ((off_bit == 1) && (cmd_bit == 1))
  if (off_bit == 1)
  {
     return 1;
  }
  return 0;

}

/*****************************************************************************
 *  See lib8113.h header file for function description                       *
 *****************************************************************************/
uint8_t dssc_checkprotocol(void)
{
   return TMode;
}

/*****************************************************************************
 *  See lib8113.h header file for function description                       *
 *****************************************************************************/
int16_t dssc_init(void)
{

	while(read_bit(CKCN,0x20));

        // Initialize Relay control pins (PO=0, PD=1, Relays off)
        configure_out(PD0,HP_VDD_DIR);
        clear_bit(PO0,HP_VDD_OUT);
        configure_out(PD0,HP_VDDA_DIR);
        clear_bit(PO0,HP_VDDA_OUT);

        // Initialize LED control pins (PO=0, PD=1, output 1)
        set_bit(PO1,LED_FNCT1_OUT);
        configure_out(PD1,LED_FNCT1_DIR);
        set_bit(PO1,LED_FNCT2_OUT);
        configure_out(PD1,LED_FNCT2_DIR);
        set_bit(PO1,LED_FNCT3_OUT);
        configure_out(PD1,LED_FNCT3_DIR);

        // Initialize user input buttons with weak pullups
        set_bit(PO0, SW1_OUT);        // Switch 1
	configure_in(PD0, SW1_DIR);
        set_bit(PO0, SW2_OUT);        // Switch 2
	configure_in(PD0, SW2_DIR);
        set_bit(PO0, SW3_OUT);        // Switch 3
	configure_in(PD0, SW3_DIR);
	set_bit(PO0, SW4_OUT);        // Switch 4
	configure_in(PD0, SW4_DIR);

        // Card powered down
	set_bit(PO6, CMDVCC_OUT);
	configure_out(PD6, CMDVCC_DIR);

	// Hold card in reset
	set_bit(PO6, RSTIN_OUT);
	configure_out(PD6, RSTIN_DIR);

	// Put weak pullup on /OFF, UART, AUX1, AUX2 signals
	set_bit(PO6, OFF_OUT);
	configure_in(PD6, OFF_DIR);
	
        set_bit(PO6, IOUC_TX_OUT);
	configure_in(PD6, IOUC_TX_DIR);
	
        set_bit(PO3, AUX1UC_OUT);
	configure_in(PD3, AUX1UC_DIR);

	set_bit(PO3, AUX2UC_OUT);
	configure_in(PD3, AUX2UC_DIR);

	// Put DS8113 in div by 8 mode
	configure_out(PD3, CLKDIV1_DIR);
	clear_bit(PO3, CLKDIV1_OUT);
	configure_out(PD3, CLKDIV2_DIR);
	clear_bit(PO3, CLKDIV2_OUT);

	// Put UART RX in input high impedance mode
	clear_bit(PO5,IOUC_RX_OUT);
	configure_in(PD5, IOUC_RX_DIR);

        //disable spi interrupts
        SPICF=0;

       return DS8113LIB_STATUS_SUCCESS;
}

/*****************************************************************************
 *  See lib8113.h header file for function description                       *
 *****************************************************************************/
int16_t dssc_getATRbuffer(uint8_t *buff, int16_t length)
{
   memcpy(buff, card_workingBuffer, (length<ATRLength)?length:ATRLength);
   return ATRLength;
}
