/*----------------------------------------------------------------------------
 *  Copyright (C) 2007 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 */
    
/** 
 * \file lib8113.h
 * \brief DS8113 Smart Card Interface Board EMV Library Module header file
 *
 * This library contains routines required to communicate with smartcards
 * via an DS8113 while conforming to:
 * ISO 7816
 * EMV 4.1
 *  \warning  The functions in this library are <b>NOT</b> multi-process 
 *            safe--that is, if you call the same method from two different 
 *            processes at the same time, the parameters to the function 
 *            may be destroyed, yielding unpredictable results.
 */

/**
* The following typedefs are used to replace lengthy datatype names in the code
*/
typedef unsigned char  uint8_t;
typedef signed short   int16_t;
typedef unsigned short uint16_t;
typedef unsigned long  uint32_t;

/** 8113 Crystal Frequency */
#define CRYSTAL_FREQUENCY_8113 16000000L

//ds8113 signal pin bit position declarations
#define CMDVCC_OUT 0x08 //PO6.3.
#define CMDVCC_IN  0x08 //PI6.3.
#define CMDVCC_DIR 0x08 //PD6.3.
#define RSTIN_OUT  0x04 //PO6.2.
#define RSTIN_DIR  0x04 //PD6.2.
#define OFF_IN     0x02 //PI6.1.
#define OFF_OUT    0x02 //PO6.1.
#define OFF_DIR    0x02 //PD6.1.
#define XTAL1_OUT  0x20 //PO6.5.
#define XTAL1_DIR  0x20 //PD6.5.
#define POWER_18V_PORADJ_OUT 0x10 //PO6.4.
#define POWER_18V_PORADJ_DIR 0x10 //PD6.4.
#define IOUC_TX_OUT    0x08 //PO5.3.
#define IOUC_TX_IN     0x08 //PO5.3.
#define IOUC_TX_DIR    0x08 //PD5.3.
#define IOUC_RX_OUT    0x04 //PO5.2.
#define IOUC_RX_DIR    0x04 //PD5.2.
#define IOUC_RX_IN     0x04 //PI5.2.
#define AUX1UC_OUT     0x80 //PO3.7.
#define AUX1UC_DIR     0x80 //PD3.7.
#define AUX1UC_IN      0x80 //PI3.7.
#define AUX2UC_OUT     0x40 //PO3.6.
#define AUX2UC_DIR     0x40 //PD3.6.
#define AUX2UC_IN      0x40 //PI3.6.
#define CLKDIV1_OUT    0x20 //PO3.5.
#define CLKDIV1_DIR    0x20 //PD3.5.
#define CLKDIV2_OUT    0x10 //PO3.4.
#define CLKDIV2_DIR    0x10 //PD3.4.
#define POWER_5V3V_OUT 0x08 //PO3.3.
#define POWER_5V3V_DIR 0x08 //PD3.3.
#define nPRES_OUT      0x04 //PO3.2.
#define nPRES_DIR      0x04 //PD3.2.
#define nPRES_IN       0x04 //PI3.2.

// LED pins on DS8113 EV Kit
#define LED_FNCT1_OUT  0x04 // PO1.2
#define LED_FNCT1_DIR  0x04 // PD1.2
#define LED_FNCT2_OUT  0x02 // PO1.1
#define LED_FNCT2_DIR  0x02 // PD1.1
#define LED_FNCT3_OUT  0x01 // PO1.0
#define LED_FNCT3_DIR  0x01 // PD1.0

// Relay Driver Pins on DS8113 EV Kit
#define HP_VDD_OUT    0x04  // PO0.2
#define HP_VDD_DIR    0x04  // PD0.2
#define HP_VDDA_OUT   0x08  // PD0.3
#define HP_VDDA_DIR   0x08  // PD0.3

// User input buttons
#define SW1_OUT        0x80 //PO0.7
#define SW1_DIR        0x80 //PD0.7
#define SW1_IN         0x80 //PI0.7
#define SW2_OUT        0x40 //PO0.6
#define SW2_DIR        0x40 //PD0.6
#define SW2_IN         0x40 //PI0.6
#define SW3_OUT        0x20 //PO0.5
#define SW3_DIR        0x20 //PD0.5
#define SW3_IN         0x20 //PI0.5
#define SW4_OUT        0x10 //PO0.4
#define SW4_DIR        0x10 //PD0.4
#define SW4_IN         0x10 //PI0.4

// System clock should be set to the crystal or input clock
// frequency.
#define SYSTEM_CLOCK  16000000L

// Smartcard clock timer reload for 3-5MHz range
// Timer is internally divided by 2 before reload.
// Reload value is off by one to match hardware definition.
// ISO UART reloads are scaled to match output clock
#if(SYSTEM_CLOCK <= 10000000L)
	#define SMARTCARD_CLK_RELOAD 1  // div by 2
	#define SMARTCARD_CLK (SYSTEM_CLOCK / 2)
#elif (SYSTEM_CLOCK <= 20000000L)
        #define SMARTCARD_CLK_RELOAD 2  // div by 4
        #define SMARTCARD_CLK (SYSTEM_CLOCK / 4)
#elif (SYSTEM_CLOCK <= 30000000)
        #define SMARTCARD_CLK_RELOAD 3  ; div by 6
        #define SMARTCARD_CLK (SYSTEM_CLOCK / 6)
        //FIXME: No defined divider
#endif

// System constants
#define POW_TWO_TWENTYTHREE     8388608
#define SYSTEM_CLOCK_CUT        SYSTEM_CLOCK/1000
#define POW_TWO_TWENTYTHREE_CUT POW_TWO_TWENTYTHREE/1000
#define SMARTCARD_BAUD          SMARTCARD_CLK/372

// Phase register for SMARTCARD_CLK/372
// when running in serial mode 3 and smod = 0
// Phase = baud * 2^23/System clock
// Phase = 9600 * 8388608/SYSTEM_CLOCK OR
// Phase = 9600 * 8389/(SYSTEM_CLOCK/1000)
//#define PHASE_SMARTCARD ((SMARTCARD_BAUD * POW_TWO_TWENTYTHREE_CUT)/SYSTEM_CLOCK_CUT)
#define PHASE_SMARTCARD 5638
//#define PHASE_SMARTCARD 4500

/** 
*  Define for using 5 Volts for Smartcard Vcc contact
* /sa #dssc_powerup
*/
#define POWERUP_5V      0

/** 
*  Define for using 3 Volts for Smartcard Vcc contact
* /sa #dssc_powerup
*/
#define POWERUP_3V      1

/** 
*  Define for using 1.8 Volts for Smartcard Vcc contact
* /sa #dssc_powerup
*/
#define POWERUP_1p8V    2

/** 
*  Define for T0 Protocol
* /sa #dssc_checkprotocol
*/
#define PROTOCOL_T0   0

/** 
*  Define for T1 Protocol
* /sa #dssc_checkprotocol
*/
#define PROTOCOL_T1   1    

/**
 *  This status code will be returned when operation is completed successfully 
 *  /sa #dssc_powerup, #dssc_warmreset,#dssc_powerdown,    #dssc_sendsblockIFSD,
 *     #dssc_checkpresence, #dssc_checkprotocol, #dssc_sendsblockIFSD
 */
#define DS8113LIB_STATUS_SUCCESS         0

/** There is mismatch between Applied Vcc voltage and Card supported Vcc voltage
 * \sa #dssc_powerup, #dssc_warmreset
 */
#define ERR_POWERUP_VOLTAGE_INVALID     -1

/** Activation Sequence is interrupted. There could be short circuit, 
 *  or Card might have been removed.
 * \sa #dssc_powerup
 */
#define ERR_POWERUP_INTERRUPTED         -2

/** Card did not return ATR bytes as per protocol timinigs.
 * \sa #POWERUP_ISO, #POWERUP_EMV, #dssc_powerup, #dssc_warmreset
 */
#define ERR_POWERUP_ATR_TIMEOUT         -3

/** Card did not return Valid ATR bytes as per selected smartcard standard(EMV or ISO).
 * \sa #POWERUP_ISO, #POWERUP_EMV, #dssc_powerup, #dssc_warmreset
 */
#define ERR_POWERUP_ATR_INVALID         -4

/** Card returned ATR with Invalid Checksum byte.
 * \sa #POWERUP_ISO, #POWERUP_EMV, #dssc_powerup, #dssc_warmreset
 */
#define ERR_POWERUP_ATR_CRC_FAILURE     -5

/** Card supported protocol is not identified yet, you may have to powerup the card first.
 * \sa #dssc_sendAPDU
 */
#define ERR_PROTOCOL_UNSUPPORTED        -6

/** Given input slot number is not supported by library.
 * \sa #dssc_selectcard, #dssc_checkpresence
 */
#define ERR_INVALID_SLOT                -7

/** Timeout occurred while waiting for data from Card
 * \sa #dssc_sendAPDU, #dssc_sendAPDUT1
 */
#define ERR_RECEIVE_TIMEOUT             -8

/** Number of parity errors during APDU transaction exceeded than 
 * Allowed number of errors as per EMV
 * \sa #dssc_sendAPDU, #dssc_sendAPDUT1
 */
#define ERR_RECEIVE_PARITY              -9

/** Number of Checksum errors during APDU transaction exceeded than 
 * Allowed number of errors as per EMV
 * NOTE: This error code is used by library internal function
 * \sa #dssc_sendAPDU, #dssc_sendAPDUT1
 */
#define ERR_RECEIVE_LRC                 -10

/** IFSD Exchange failed with card.
 * \sa #dssc_sendAPDU, #dssc_sendAPDUT1, #dssc_sendsblockIFSD
 */
#define ERR_SETIFSD_FAILURE             -13

/** Warm reset failed
 * \sa #dssc_warmreset
 */
#define ERR_DO_WARM_RESET               -15

/** Card returned invalid Status bytes in APDU response
 * \sa #dssc_sendAPDU, #dssc_sendAPDUT0
 */
#define ERR_INVALID_STATUS_BYTE         -16

/** Unclassified error occurred
 * \sa #dssc_sendAPDU, #dssc_sendAPDUT0, #dssc_sendAPDUT1
 */
#define ERR_UNKNOWN_ERROR               -17

/** T1 Transaction is aborted. 
 * \sa #dssc_sendAPDU, #dssc_sendAPDUT1
 */
#define ERR_ABORT_T1_TRANSACTION        -20

/** The Number of errors exceeded than allowed number of error while t1 transaction
 * \sa #dssc_sendAPDU, #dssc_sendAPDUT1
 */
#define ERR_MAX_RETRIED                 -21

/** Card returned invalid NAD
 * NOTE: This error code is used by library internal function
 * \sa #dssc_sendAPDU, #dssc_sendAPDUT1
 */
#define ERR_INVALID_NAD                 -22

/** Card returned invalid Length.
 * NOTE: This error code is used by library internal function
 * \sa #dssc_sendAPDU, #dssc_sendAPDUT1
 */
#define ERR_INVALID_LEN                 -23

/** INF field in received T1 block from card is invalid. 
 * NOTE: This error code is used by library internal function
 * \sa #dssc_sendAPDU, #dssc_sendAPDUT1
 */
#define ERR_INVALID_INF                 -24


#define ERR_FRAMING_ERROR               -25 


/** Version number associated with this header file. Should be the same as
 * the version number returned by the <i>#dssc_version</i> function.
 * \sa #dssc_version */
#define DSSC_VERSION      "1.00.00";

/**
 * \brief   Returns DS8113 Smart Card Interface Board EMV Library Version information
 *
 * \return  Version string
 */
extern char *dssc_version(void);

/**
 * \brief   Power up the currently selected smartcard.
 * \param   voltage Powerup voltage to use (POWERUP_5V, POWERUP3V or POWERUP_1p8V)
 * \return  <i>#DS8113LIB_STATUS_SUCCESS</i> if the operation is completed successfully,
 *           Otherwise, It returns, one of the following error codes:
 *          #ERR_POWERUP_VOLTAGE_INVALID, #ERR_POWERUP_INTERRUPTED,
 *          #ERR_POWERUP_ATR_INVALID, #ERR_POWERUP_ATR_TIMEOUT,
 *          #ERR_POWERUP_ATR_CRC_FAILURE, #ERR_DO_WARM_RESET, #ERR_FRAMING_ERROR
 *
 */
int16_t dssc_powerup(uint8_t voltage);

/* Added by FVT 2/01/08 * * * * * * * * * * */ 
int16_t dssc_powerdown(void);
int16_t dssc_init(void);
int16_t dssc_checkpresence(void);
void start_clock(void);
void conf_timer21_etu_timeout_value(void);
int dssc_read_first_atr_byte(void);
int dssc_read_atr_byte(void);

/**
 * \brief   Warm reset the currently selected smartcard.
 * \return  <i>#DS8007LIB_STATUS_SUCCESS</i> if the operation is completed successfully,
 *           Otherwise, It returns, one of the following error codes:
 *     #ERR_POWERUP_VOLTAGE_INVALID, #ERR_POWERUP_INTERRUPTED,
 *          #ERR_POWERUP_ATR_INVALID,#ERR_POWERUP_ATR_TIMEOUT,
 *          #ERR_POWERUP_ATR_CRC_FAILURE,#ERR_DO_WARM_RESET, #ERR_FRAMING_ERROR
 */
int16_t dssc_warmreset(void);


/**
 * \brief   returns active protocol used for given slot.
 * \return  PROTOCOL_T0 for T0 protcol, PROTOCL_T1 for t1 protocol
 */
uint8_t dssc_checkprotocol(void);

/**
 * \brief   Retrieve the ATR captured during the last dssc_powerup() call.
 * \param   buff Buffer to store ATR.
 * \param   length Maximum length to copy into buffer.
 * \return  Length of ATR buffer returned
 */
int16_t dssc_getATRbuffer(uint8_t *buff,int16_t length);

/**
 * \brief   Retrieve the ATR structure of the currently selected card.
 * \param   userATR Struct in which to store ATR information
 */
void dssc_getATR(struct ATR *userATR);

/**
 * \brief   Send an APDU to currently selected smartcard.
 *
 * Will choose to call either dssc_sendAPDUT0 or T1 based on TD
 * protocol selection of ATR.
 * 
 * \param   buffer Holds the Command APDU to send
 * \param   length Length of Command APDU
 * \param   rbuffer Location to put Response APDU
 * \return  Length of APDU buffer returned, 
 * Otherwise, returns one of the following error codes:
 *        #ERR_RECEIVE_TIMEOUT
 *        #ERR_RECEIVE_PARITY
 *        #ERR_INVALID_STATUS_BYTE
 *        #ERR_UNKNOWN_ERROR
 *        #ERR_ABORT_T1_TRANSACTION
 *        #ERR_MAX_RETRIED 
 *
 */
int16_t dssc_sendAPDU(uint8_t *buffer,uint16_t length,uint8_t *rbuffer);

/**
 * \brief   Send an APDU using protocol T=0 to currently selected smartcard.
 * \param   buffer Holds the Command APDU to send
 * \param   length Length of Command APDU
 * \param   rbuffer Location to put Response APDU
 * \return  Length of APDU buffer returned, 
 * Otherwise, returns one of the following error codes:
 *          #ERR_RECEIVE_TIMEOUT
 *          #ERR_RECEIVE_PARITY
 *          #ERR_INVALID_STATUS_BYTE
 *          #ERR_UNKNOWN_ERROR 
 */
int16_t dssc_sendAPDUT0(uint8_t *buffer,uint16_t length,uint8_t *rbuffer);

/**
  * \brief   Send an APDU using protocol T=1 to currently selected smartcard.
  * \param   buffer Holds the Command APDU to send
  * \param   length Length of Command APDU
  * \param   rbuffer Location to put Response APDU
  * \return  Length of APDU buffer returned, 
  * Otherwise, returns one of the following error codes:
  *        #ERR_INVALID_STATUS_BYTE
  *        #ERR_UNKNOWN_ERROR 
  *        #ERR_ABORT_T1_TRANSACTION 
  *        #ERR_MAX_RETRIED 
  *
  */
int16_t dssc_sendAPDUT1(uint8_t *buffer,uint16_t length,uint8_t *rbuffer);

/**
  * \brief   Send an sblock with IFSD parameter.  (for T=1 communication only)
  * \param   IFSD value to use
  *
  * \return  <i>#DS8007LIB_STATUS_SUCCESS</i> if the operation is completed successfully,
  *           Otherwise, It returns #ERR_SETIFSD_FAILURE
  */
int16_t dssc_sendsblockIFSD(uint8_t IFSD);

/**
  * \brief   Set node address parameter.  (for T=1 communication only)
  * \param   value NAD value to use
  */
void dssc_setNAD(uint8_t value);

void stopbit_interrupt(void);
